"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var DataSourceIndexed_1 = require("./DataSourceIndexed");
var StringExtensions_1 = require("../Utilities/Extensions/StringExtensions");
var ExpressionHelper_1 = require("../Utilities/Helpers/ExpressionHelper");
var Enums_1 = require("../PredefinedConfig/Common/Enums");
var ArrayExtensions_1 = require("../Utilities/Extensions/ArrayExtensions");
var RangeHelper_1 = require("../Utilities/Helpers/RangeHelper");
/* There are 3 possible reasons why we might need to filter / search the grid:
1. If there is an Advanced Search (i.e. CurrentAdvancedSearch is not empty)
2. If there are any Column Filters applied
3. If Quick Search has been applied (though it could be that the action is HighlightCell in which case we dont need to do any filtering)
We also need to take account of what the server search option is, as that will affect our actions
*/
//All custom pipelines should extend from pipelineBase
exports.FilterAndSearchDataSource = function (blotter) {
    return DataSourceIndexed_1.DataSourceIndexed.extend('FilterAndSearchDataSource', {
        blotter: blotter,
        apply: function () {
            // before we start we will clear any cell highlights that are the result of Quick Search - as we will apply that at the end if required
            this.clearColorQuickSearch();
            // Lets first get the 3 variables that indicate a filter/search is required
            var currentSearchName = blotter.api.advancedSearchApi.getCurrentAdvancedSearchName();
            var columnFilters = blotter.api.columnFilterApi.getAllColumnFilter();
            var quickSearchText = blotter.api.quickSearchApi.getQuickSearchValue();
            // If any of these 3 are set then we need to build the index and color the quick search; otherwise we should clear it
            if (StringExtensions_1.StringExtensions.IsNotNullOrEmpty(currentSearchName) ||
                ArrayExtensions_1.ArrayExtensions.IsNotNullOrEmpty(columnFilters) ||
                StringExtensions_1.StringExtensions.IsNotNullOrEmpty(quickSearchText)) {
                this.buildIndex(this.filterTest);
                this.colorQuickSearch();
            }
            else {
                this.clearIndex();
            }
        },
        /*
        This boolean function will test for each row whether it passes ALL the possible filter / search requirements
        Only rows that pass all 3 (i.e. Advanced Search, Column Filters, Quick Search) will return true
        */
        filterTest: function (r, rowObject) {
            var e_1, _a, e_2, _b;
            var columns = blotter.api.gridApi.getColumns();
            // if no columns then nothing to filter
            if (ArrayExtensions_1.ArrayExtensions.IsNullOrEmpty(columns)) {
                return true;
            }
            var serverSearchOption = blotter.blotterOptions.generalOptions.serverSearchOption;
            // first let's assess ADVANCED SEARCH
            // Note: serverSearchOption has to be 'None' becasue any other value then they are performing search on the server and nothing for us to do
            if (serverSearchOption == 'None') {
                var currentSearchName_1 = blotter.api.advancedSearchApi.getCurrentAdvancedSearchName();
                if (StringExtensions_1.StringExtensions.IsNotNullOrEmpty(currentSearchName_1)) {
                    // Get the actual Advanced Search object and check it exists
                    var currentSearch = blotter.api.advancedSearchApi
                        .getAllAdvancedSearch()
                        .find(function (s) { return s.Name == currentSearchName_1; });
                    if (currentSearch && currentSearch.Expression) {
                        // See if our record passes the Advanced Search Expression - using Expression Helper; if not then return false
                        if (!ExpressionHelper_1.ExpressionHelper.checkForExpressionFromRecord(currentSearch.Expression, rowObject, columns, blotter)) {
                            return false;
                        }
                    }
                }
            }
            // now let's assess COLUMN FILTERS to see if our record passes
            // NOTE: serverSearchOption has to be 'None' or 'AdvancedSearch' because any other value then they are checking filters on the server and nothing for us to do
            if (serverSearchOption == 'None' || serverSearchOption == 'AdvancedSearch') {
                // Get the column filters
                var columnFilters = blotter.api.columnFilterApi.getAllColumnFilter();
                if (ArrayExtensions_1.ArrayExtensions.IsNotNullOrEmpty(columnFilters)) {
                    try {
                        for (var columnFilters_1 = tslib_1.__values(columnFilters), columnFilters_1_1 = columnFilters_1.next(); !columnFilters_1_1.done; columnFilters_1_1 = columnFilters_1.next()) {
                            var columnFilter = columnFilters_1_1.value;
                            if (columnFilter.Filter) {
                                // See if our record passes the Filter Expression - using Expression Helper; if not then return false
                                if (!ExpressionHelper_1.ExpressionHelper.checkForExpressionFromRecord(columnFilter.Filter, rowObject, columns, blotter)) {
                                    return false;
                                }
                            }
                        }
                    }
                    catch (e_1_1) { e_1 = { error: e_1_1 }; }
                    finally {
                        try {
                            if (columnFilters_1_1 && !columnFilters_1_1.done && (_a = columnFilters_1.return)) _a.call(columnFilters_1);
                        }
                        finally { if (e_1) throw e_1.error; }
                    }
                }
                // finally, let's assess QUICK SEARCH
                var quickSearchState = blotter.api.quickSearchApi.getQuickSearchState();
                // check that we have quick search running
                var range = RangeHelper_1.RangeHelper.CreateValueRangeFromOperand(quickSearchState.QuickSearchText);
                if (range != null) {
                    // with quick search because we need to colour and might not need to filter we dont return true/false but instead set a return value
                    var recordReturnValue = false;
                    var rowId = blotter.getPrimaryKeyValueFromRecord(rowObject);
                    try {
                        // only check on visible columns for quick search
                        for (var _c = tslib_1.__values(columns.filter(function (c) { return c.Visible; })), _d = _c.next(); !_d.done; _d = _c.next()) {
                            var column = _d.value;
                            var isMatch = false;
                            if (RangeHelper_1.RangeHelper.IsColumnAppropriateForRange(range.Operator, column)) {
                                var expression = ExpressionHelper_1.ExpressionHelper.CreateSingleColumnExpression(column.ColumnId, null, null, null, [range]);
                                if (ExpressionHelper_1.ExpressionHelper.checkForExpressionFromRecord(expression, rowObject, [column], blotter)) {
                                    isMatch = true;
                                }
                            }
                            if (isMatch) {
                                //if we need to display ONLY the rows that matched the quicksearch and dont need to colour them then we can return true
                                if (quickSearchState.DisplayAction == Enums_1.DisplayAction.ShowRow) {
                                    return true;
                                }
                                else {
                                    // if we need to color then cell then add it to the collection otherwise we add undefined so we clear previous properties
                                    this.quickSearchColorCollection.push({
                                        rowID: rowId,
                                        columnId: column.ColumnId,
                                        style: { quickSearchStyle: quickSearchState.Style },
                                    });
                                }
                                // set the return value to true (but dont return immediately as we might need to colour other cells in the row)
                                recordReturnValue = true;
                            }
                        }
                    }
                    catch (e_2_1) { e_2 = { error: e_2_1 }; }
                    finally {
                        try {
                            if (_d && !_d.done && (_b = _c.return)) _b.call(_c);
                        }
                        finally { if (e_2) throw e_2.error; }
                    }
                    //if we color only then we just return true (as we have now built the collection of cells to colour)....
                    if (quickSearchState.DisplayAction == Enums_1.DisplayAction.HighlightCell) {
                        return true;
                    }
                    return recordReturnValue;
                }
            }
            return true;
        },
        /* If any of the 3 causes of searching are present then return the index;
        otherwise return the row count of the data source.
        */
        getRowCount: function () {
            var currentSearchName = blotter.api.advancedSearchApi.getCurrentAdvancedSearchName();
            var columnFilters = blotter.api.columnFilterApi.getAllColumnFilter();
            var quickSearchText = blotter.api.quickSearchApi.getQuickSearchValue();
            if (StringExtensions_1.StringExtensions.IsNotNullOrEmpty(currentSearchName) ||
                ArrayExtensions_1.ArrayExtensions.IsNotNullOrEmpty(columnFilters) ||
                StringExtensions_1.StringExtensions.IsNotNullOrEmpty(quickSearchText)) {
                return this.index.length;
            }
            else {
                return this.dataSource.getRowCount();
            }
        },
        colorQuickSearch: function () {
            var e_3, _a;
            try {
                for (var _b = tslib_1.__values(this.quickSearchColorCollection), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var record = _c.value;
                    blotter.addCellStyleHypergrid(record.rowID, record.columnId, record.style);
                }
            }
            catch (e_3_1) { e_3 = { error: e_3_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                }
                finally { if (e_3) throw e_3.error; }
            }
        },
        clearColorQuickSearch: function () {
            if (this.quickSearchColorCollection) {
                blotter.removeAllCellStyleHypergrid('QuickSearch');
            }
            this.quickSearchColorCollection = [];
        },
    });
};
