"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Enums_1 = require("../PredefinedConfig/Common/Enums");
var StringExtensions_1 = require("../Utilities/Extensions/StringExtensions");
var MenuItem_1 = require("../Utilities/MenuItem");
/**
 * Base class for all strategies and does most of the work of creating menus
 * Each strategy is reponsible for managing state (through InitState())
 */
var AdaptableStrategyBase = /** @class */ (function () {
    function AdaptableStrategyBase(Id, blotter) {
        this.Id = Id;
        this.blotter = blotter;
        this.Id = Id;
        this.blotter = blotter;
    }
    AdaptableStrategyBase.prototype.initializeWithRedux = function () {
        var _this = this;
        this.InitState();
        this.blotter.adaptableBlotterStore.TheStore.subscribe(function () { return _this.InitState(); });
    };
    AdaptableStrategyBase.prototype.InitState = function () {
        /**
         *  derived in each strategy that needs to manage state
         * Most now have been taken elsewhere into services.  All that is left is:
         *  Chart Strategy - lots
         *  Custom Sort
         *  Percent Bar
         *  Theme
         */
    };
    AdaptableStrategyBase.prototype.getPopupMenuItem = function () {
        if (!this.hasPopupMenu()) {
            return null;
        }
        if (this.popupMenuItem == null) {
            this.addPopupMenuItem();
        }
        return this.popupMenuItem;
    };
    AdaptableStrategyBase.prototype.hasPopupMenu = function () {
        return true;
    };
    AdaptableStrategyBase.prototype.addPopupMenuItem = function () {
        // base class implementation which is empty
    };
    AdaptableStrategyBase.prototype.addColumnMenuItem = function (column) {
        // base class implementation which is empty
    };
    AdaptableStrategyBase.prototype.getStrategyEntitlement = function () {
        var _this = this;
        var functionEntitlements = this.blotter.api.entitlementApi.getEntitlementState()
            .FunctionEntitlements;
        return functionEntitlements.find(function (x) { return x.FunctionName == _this.Id; });
    };
    AdaptableStrategyBase.prototype.isVisibleStrategy = function () {
        var entitlement = this.getStrategyEntitlement();
        if (entitlement) {
            return entitlement.AccessLevel != 'Hidden';
        }
        return true;
    };
    AdaptableStrategyBase.prototype.isReadOnlyStrategy = function () {
        var entitlement = this.getStrategyEntitlement();
        if (entitlement) {
            if (entitlement.AccessLevel == 'ReadOnly') {
                return true;
            }
            else {
                return false;
            }
        }
        return false;
    };
    // creates the menu items in the main dropdown
    AdaptableStrategyBase.prototype.createMenuItemShowPopup = function (Label, ComponentName, GlyphIcon, PopupParams) {
        var menuItemShowPopup = new MenuItem_1.MenuItemShowPopup(Label, this.Id, ComponentName, GlyphIcon, this.isVisibleStrategy(), PopupParams);
        this.popupMenuItem = menuItemShowPopup;
    };
    // direct actions called by the column menu - invisible if strategy is hidden or readonly
    AdaptableStrategyBase.prototype.createColumnMenuItemReduxAction = function (Label, GlyphIcon, Action) {
        if (this.isVisibleStrategy() && !this.isReadOnlyStrategy()) {
            var menuItemShowPopup = new MenuItem_1.MenuItemDoReduxAction(Label, this.Id, Action, GlyphIcon, true);
            this.addMenuItemToStore(menuItemShowPopup);
        }
    };
    // popups called by the column menu - invisible if strategy is hidden or readonly
    AdaptableStrategyBase.prototype.createColumnMenuItemShowPopup = function (Label, ComponentName, GlyphIcon, PopupParams) {
        if (this.isVisibleStrategy() && !this.isReadOnlyStrategy()) {
            var menuItemShowPopup = new MenuItem_1.MenuItemShowPopup(Label, this.Id, ComponentName, GlyphIcon, true, PopupParams);
            this.addMenuItemToStore(menuItemShowPopup);
        }
    };
    AdaptableStrategyBase.prototype.addMenuItemToStore = function (menuItem) {
        this.blotter.api.internalApi.addColumnMenuItem(menuItem);
    };
    AdaptableStrategyBase.prototype.canCreateColumnMenuItem = function (column, blotter, functionType) {
        if (functionType === void 0) { functionType = ''; }
        if (this.isReadOnlyStrategy()) {
            return false;
        }
        if (StringExtensions_1.StringExtensions.IsNotNullOrEmpty(functionType)) {
            if (functionType == 'sort') {
                return column.Sortable;
            }
            else if (functionType == 'numeric') {
                return column.DataType == Enums_1.DataType.Number;
            }
            else if (functionType == 'columnfilter') {
                return column.Filterable;
            }
            else if (functionType == 'quickfilter') {
                return (blotter.hasQuickFilter &&
                    blotter.blotterOptions.filterOptions.useAdaptableBlotterQuickFilter);
            }
        }
        return true;
    };
    return AdaptableStrategyBase;
}());
exports.AdaptableStrategyBase = AdaptableStrategyBase;
