"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var LoggingHelper_1 = require("../Helpers/LoggingHelper");
var AuditLogEntry_1 = require("../Interface/AuditLogEntry");
var ColumnHelper_1 = require("../Helpers/ColumnHelper");
var AuditLogService = /** @class */ (function () {
    function AuditLogService(blotter) {
        var _this = this;
        this.canSendLog = true;
        this.numberOfMissedPing = 0;
        this.auditLogQueue = [];
        this.blotter = blotter;
        // Internal State
        if (blotter.blotterOptions.auditOptions != null &&
            blotter.blotterOptions.auditOptions.auditInternalStateChanges != null) {
            this.isAuditInternalStateChangesEnabled = this.isAuditOptionEnabled(blotter.blotterOptions.auditOptions.auditInternalStateChanges);
        }
        else {
            this.isAuditInternalStateChangesEnabled = false;
        }
        // User State
        if (blotter.blotterOptions.auditOptions != null &&
            blotter.blotterOptions.auditOptions.auditUserStateChanges != null) {
            this.isAuditUserStateChangesEnabled = this.isAuditOptionEnabled(blotter.blotterOptions.auditOptions.auditUserStateChanges);
        }
        else {
            this.isAuditUserStateChangesEnabled = false;
        }
        // Function Events
        if (blotter.blotterOptions.auditOptions != null &&
            blotter.blotterOptions.auditOptions.auditFunctionEvents != null) {
            this.isAuditFunctionEventsEnabled = this.isAuditOptionEnabled(blotter.blotterOptions.auditOptions.auditFunctionEvents);
        }
        else {
            this.isAuditFunctionEventsEnabled = false;
        }
        // Cell Edit
        if (blotter.blotterOptions.auditOptions != null &&
            blotter.blotterOptions.auditOptions.auditCellEdits != null) {
            this.isAuditCellEditsEnabled = this.isAuditOptionEnabled(blotter.blotterOptions.auditOptions.auditCellEdits);
        }
        else {
            this.isAuditCellEditsEnabled = false;
        }
        // Ticking Data Changes
        if (blotter.blotterOptions.auditOptions != null &&
            blotter.blotterOptions.auditOptions.auditTickingDataChanges != null) {
            this.isAuditTickingDataChangesEnabled = this.isAuditOptionEnabled(blotter.blotterOptions.auditOptions.auditTickingDataChanges);
        }
        else {
            this.isAuditTickingDataChangesEnabled = false;
        }
        // Log State
        this.isAuditStateChangesEnabled =
            this.isAuditInternalStateChangesEnabled || this.isAuditUserStateChangesEnabled;
        // General Audit Flag
        this.isAuditEnabled =
            this.isAuditStateChangesEnabled ||
                this.isAuditFunctionEventsEnabled ||
                this.isAuditCellEditsEnabled;
        // set up the listener if we are auditing Data Changes
        if (this.isAuditTickingDataChangesEnabled) {
            blotter.DataService.OnDataSourceChanged().Subscribe(function (sender, eventText) {
                return _this.handleDataSourceChanged(eventText);
            });
        }
        // set up the Audit Queue if any of the Audits is set to use HTTP Channel
        if (this.isAuditEnabled) {
            if (blotter.blotterOptions.auditOptions != undefined) {
                if (this.shouldAuditToHttpChannel(blotter.blotterOptions.auditOptions)) {
                    if (blotter.blotterOptions.auditOptions.pingInterval != undefined &&
                        blotter.blotterOptions.auditOptions.auditLogsSendInterval != undefined) {
                        this.ping();
                        setInterval(function () { return _this.ping(); }, blotter.blotterOptions.auditOptions.pingInterval * 1000);
                        setInterval(function () { return _this.flushAuditQueue(); }, blotter.blotterOptions.auditOptions.auditLogsSendInterval * 1000);
                    }
                }
            }
        }
    }
    AuditLogService.prototype.addEditCellAuditLogBatch = function (dataChangedEvents) {
        var _this = this;
        dataChangedEvents.forEach(function (dce) {
            _this.addEditCellAuditLog(dce);
        });
    };
    AuditLogService.prototype.addEditCellAuditLog = function (dataChangedInfo) {
        if (this.isAuditCellEditsEnabled) {
            var auditLogEntry = this.createAuditLogEntryFromDataChangedInfo(dataChangedInfo, AuditLogEntry_1.AuditLogType.CellEdit);
            var auditDestinationOptions = this.blotter.blotterOptions.auditOptions.auditCellEdits;
            if (auditDestinationOptions.auditToConsole) {
                LoggingHelper_1.LoggingHelper.LogObject(auditLogEntry);
            }
            if (auditDestinationOptions.auditAsEvent) {
                this.publishStateChanged(auditLogEntry, AuditLogEntry_1.AuditLogType.CellEdit);
            }
            if (auditDestinationOptions.auditToHttpChannel) {
                this.auditLogQueue.push(auditLogEntry);
            }
            if (auditDestinationOptions.auditAsAlert) {
                var message = 'Column: ' +
                    ColumnHelper_1.default.getFriendlyNameFromColumnId(dataChangedInfo.ColumnId, this.blotter.api.gridApi.getColumns()) +
                    '.  Identifier: ' +
                    dataChangedInfo.IdentifierValue +
                    '.  Old Value: ' +
                    dataChangedInfo.OldValue +
                    '.  New Value: ' +
                    dataChangedInfo.NewValue;
                this.showAlert('Data Changed', message);
            }
        }
    };
    AuditLogService.prototype.addUserStateChangeAuditLog = function (stateChangeDetails) {
        if (this.isAuditUserStateChangesEnabled) {
            var auditLogEntry = {
                auditlog_type: AuditLogEntry_1.AuditLogType.UserStateChange,
                client_timestamp: new Date(),
                username: this.blotter.blotterOptions.userName,
                blotter_id: this.blotter.blotterOptions.blotterId,
                state_change_details: stateChangeDetails,
            };
            var auditDestinationOptions = this.blotter.blotterOptions.auditOptions
                .auditUserStateChanges;
            if (auditDestinationOptions.auditToConsole) {
                LoggingHelper_1.LoggingHelper.LogObject(auditLogEntry);
            }
            if (auditDestinationOptions.auditAsEvent) {
                this.publishStateChanged(auditLogEntry, AuditLogEntry_1.AuditLogType.UserStateChange);
            }
            if (auditDestinationOptions.auditToHttpChannel) {
                this.auditLogQueue.push(auditLogEntry);
            }
            if (auditDestinationOptions.auditAsAlert) {
                var message = 'Action: ' + stateChangeDetails.actionType;
                this.showAlert(stateChangeDetails.name + ' StateChange', message);
            }
        }
    };
    AuditLogService.prototype.addInternalStateChangeAuditLog = function (stateChangeDetails) {
        if (this.isAuditInternalStateChangesEnabled) {
            var auditLogEntry = {
                auditlog_type: AuditLogEntry_1.AuditLogType.InternalStateChange,
                client_timestamp: new Date(),
                username: this.blotter.blotterOptions.userName,
                blotter_id: this.blotter.blotterOptions.blotterId,
                state_change_details: stateChangeDetails,
            };
            var auditDestinationOptions = this.blotter.blotterOptions.auditOptions
                .auditInternalStateChanges;
            if (auditDestinationOptions.auditToConsole) {
                LoggingHelper_1.LoggingHelper.LogObject(auditLogEntry);
            }
            if (auditDestinationOptions.auditAsEvent) {
                this.publishStateChanged(auditLogEntry, AuditLogEntry_1.AuditLogType.InternalStateChange);
            }
            if (auditDestinationOptions.auditToHttpChannel) {
                this.auditLogQueue.push(auditLogEntry);
            }
            if (auditDestinationOptions.auditAsAlert) {
                var message = 'Action: ' + stateChangeDetails.actionType;
                this.showAlert(stateChangeDetails.name + ' StateChange', message);
            }
        }
    };
    AuditLogService.prototype.addFunctionAppliedAuditLog = function (functionAppliedDetails) {
        if (this.isAuditFunctionEventsEnabled) {
            var auditLogEntry = {
                auditlog_type: AuditLogEntry_1.AuditLogType.FunctionApplied,
                client_timestamp: new Date(),
                username: this.blotter.blotterOptions.userName,
                blotter_id: this.blotter.blotterOptions.blotterId,
                function_applied_details: functionAppliedDetails,
            };
            var auditDestinationOptions = this.blotter.blotterOptions.auditOptions.auditFunctionEvents;
            if (auditDestinationOptions.auditToConsole) {
                LoggingHelper_1.LoggingHelper.LogObject(auditLogEntry);
            }
            if (auditDestinationOptions.auditAsEvent) {
                this.publishStateChanged(auditLogEntry, AuditLogEntry_1.AuditLogType.FunctionApplied);
            }
            if (auditDestinationOptions.auditToHttpChannel) {
                this.auditLogQueue.push(auditLogEntry);
            }
            if (auditDestinationOptions.auditAsAlert) {
                var message = 'Action: ' + functionAppliedDetails.action + '.  Details: ' + functionAppliedDetails.info;
                this.showAlert(functionAppliedDetails.name + ' Function Applied', message);
            }
        }
    };
    AuditLogService.prototype.handleDataSourceChanged = function (dataChangedInfo) {
        if (this.isAuditTickingDataChangesEnabled) {
            var auditLogEntry = this.createAuditLogEntryFromDataChangedInfo(dataChangedInfo, AuditLogEntry_1.AuditLogType.TickingDataChange);
            var auditDestinationOptions = this.blotter.blotterOptions.auditOptions
                .auditTickingDataChanges;
            if (auditDestinationOptions.auditToConsole) {
                LoggingHelper_1.LoggingHelper.LogObject(auditLogEntry);
            }
            if (auditDestinationOptions.auditAsEvent) {
                this.publishStateChanged(auditLogEntry, AuditLogEntry_1.AuditLogType.FunctionApplied);
            }
            if (auditDestinationOptions.auditToHttpChannel) {
                this.auditLogQueue.push(auditLogEntry);
            }
            if (auditDestinationOptions.auditAsAlert) {
                var message = 'Column: ' +
                    ColumnHelper_1.default.getFriendlyNameFromColumnId(dataChangedInfo.ColumnId, this.blotter.api.gridApi.getColumns()) +
                    '.  Identifier: ' +
                    dataChangedInfo.IdentifierValue +
                    '.  Old Value: ' +
                    dataChangedInfo.OldValue +
                    '.  New Value: ' +
                    dataChangedInfo.NewValue;
                this.showAlert('Data Changed', message);
            }
        }
    };
    AuditLogService.prototype.createAuditLogEntryFromDataChangedInfo = function (dataChangedInfo, auditLogType) {
        return {
            auditlog_type: auditLogType,
            client_timestamp: new Date(),
            username: this.blotter.blotterOptions.userName,
            blotter_id: this.blotter.blotterOptions.blotterId,
            ticking_data_change_details: {
                primarykey_column_value: String(dataChangedInfo.IdentifierValue),
                primarykey_column_id: this.blotter.blotterOptions.primaryKey,
                column_id: dataChangedInfo.ColumnId,
                previous_value: String(dataChangedInfo.OldValue),
                new_value: String(dataChangedInfo.NewValue),
                row_data: this.blotter.getDataRowFromRecord(dataChangedInfo.Record),
            },
        };
    };
    AuditLogService.prototype.ping = function () {
        var _this = this;
        var pingMessage = {
            auditlog_type: AuditLogEntry_1.AuditLogType.Ping,
            client_timestamp: new Date(),
            username: this.blotter.blotterOptions.userName,
            blotter_id: this.blotter.blotterOptions.blotterId,
            number_of_missed_ping: this.numberOfMissedPing,
        };
        var xhr = new XMLHttpRequest();
        xhr.onerror = function (ev) {
            LoggingHelper_1.LoggingHelper.LogAdaptableBlotterError('error sending ping: ' + ev.message);
            _this.SetCanSendLog(false);
        };
        xhr.ontimeout = function (ev) {
            LoggingHelper_1.LoggingHelper.LogAdaptableBlotterError('timeout sending ping');
            _this.SetCanSendLog(false);
        };
        xhr.onload = function (ev) {
            if (xhr.readyState == 4) {
                if (xhr.status == 200) {
                    _this.SetCanSendLog(true);
                }
                else {
                    LoggingHelper_1.LoggingHelper.LogAdaptableBlotterError('error sending ping: ' + xhr.statusText);
                    _this.SetCanSendLog(false);
                }
            }
        };
        var url = '/auditlog';
        xhr.open('POST', url, true);
        xhr.setRequestHeader('Content-type', 'application/json');
        xhr.send(JSON.stringify(pingMessage));
    };
    AuditLogService.prototype.SetCanSendLog = function (enable) {
        if (enable) {
            this.canSendLog = true;
            this.numberOfMissedPing = 0;
        }
        else {
            this.canSendLog = false;
            this.numberOfMissedPing++;
        }
    };
    AuditLogService.prototype.flushAuditQueue = function () {
        //if we cannot send logs then we just clear the thing
        if (!this.canSendLog) {
            this.auditLogQueue.length = 0;
        }
        var obj = this.auditLogQueue.shift();
        var _loop_1 = function () {
            var xhr = new XMLHttpRequest();
            xhr.onerror = function (ev) {
                return LoggingHelper_1.LoggingHelper.LogAdaptableBlotterError('error sending AuditLog: ' + ev.message);
            };
            xhr.ontimeout = function (pe) {
                return LoggingHelper_1.LoggingHelper.LogAdaptableBlotterError('timeout sending AuditLog');
            };
            xhr.onload = function (pe) {
                if (xhr.readyState == 4) {
                    if (xhr.status != 200) {
                        LoggingHelper_1.LoggingHelper.LogAdaptableBlotterError('error sending AuditLog: ' + xhr.statusText);
                    }
                }
            };
            url = '/auditlog';
            //we make the request async
            xhr.open('POST', url, true);
            xhr.setRequestHeader('Content-type', 'application/json');
            xhr.send(JSON.stringify(obj));
            obj = this_1.auditLogQueue.shift();
        };
        var this_1 = this, url;
        // while (obj && this.sockJS.readyState == SockJS.OPEN) {
        while (obj) {
            _loop_1();
        }
    };
    AuditLogService.prototype.convertAuditMessageToText = function (obj) {
        var stringArray = [];
        if (obj == undefined) {
            return String(obj);
        }
        else if (Array.isArray(obj)) {
            for (var prop in obj) {
                stringArray.push(this.convertAuditMessageToText(obj[prop]));
            }
            return '[' + stringArray.join(',') + ']';
        }
        if (typeof obj == 'object') {
            for (var prop in obj) {
                if (obj.hasOwnProperty(prop)) {
                    stringArray.push(prop + ': ' + this.convertAuditMessageToText(obj[prop]));
                }
            }
            return '{' + stringArray.join(',') + '}';
            //is function
        }
        else if (typeof obj == 'function') {
            stringArray.push(obj.toString());
        }
        else {
            stringArray.push(String(obj));
        }
        return stringArray.join(',');
    };
    AuditLogService.prototype.isAuditOptionEnabled = function (auditDestinationOptions) {
        return (auditDestinationOptions.auditAsEvent ||
            auditDestinationOptions.auditToConsole ||
            auditDestinationOptions.auditToHttpChannel ||
            auditDestinationOptions.auditAsAlert);
    };
    AuditLogService.prototype.shouldAuditToHttpChannel = function (auditLogOptions) {
        if (auditLogOptions) {
            if (auditLogOptions.auditCellEdits) {
                if (auditLogOptions.auditCellEdits.auditToHttpChannel) {
                    return true;
                }
            }
            if (auditLogOptions.auditFunctionEvents) {
                if (auditLogOptions.auditFunctionEvents.auditToHttpChannel) {
                    return true;
                }
            }
            if (auditLogOptions.auditInternalStateChanges) {
                if (auditLogOptions.auditInternalStateChanges.auditToHttpChannel) {
                    return true;
                }
            }
            if (auditLogOptions.auditUserStateChanges) {
                if (auditLogOptions.auditUserStateChanges.auditToHttpChannel) {
                    return true;
                }
            }
        }
        return false;
    };
    AuditLogService.prototype.showAlert = function (header, message) {
        var alert = {
            Header: header,
            Msg: message,
            MessageType: this.blotter.blotterOptions.auditOptions.alertMessageType,
            ShowAsPopup: this.blotter.blotterOptions.auditOptions.alertShowAsPopup,
        };
        this.blotter.api.alertApi.displayAlert(alert);
    };
    AuditLogService.prototype.publishStateChanged = function (auditLogEntry, auditLogType) {
        var stateEventData = {
            name: 'Adaptable Blotter',
            type: 'Audit Log Event',
            id: auditLogEntry,
        };
        var stateChangedArgs = {
            object: 'fdc3-context',
            definition: 'https://fdc3.org/context/1.0.0/',
            version: '1.0.0',
            data: [stateEventData],
        };
        switch (auditLogType) {
            case AuditLogEntry_1.AuditLogType.CellEdit:
                this.blotter.api.auditEventApi._onAuditCellEdited.Dispatch(this.blotter, stateChangedArgs);
                break;
            case AuditLogEntry_1.AuditLogType.FunctionApplied:
                this.blotter.api.auditEventApi._onAuditFunctionApplied.Dispatch(this.blotter, stateChangedArgs);
                break;
            case AuditLogEntry_1.AuditLogType.InternalStateChange:
            case AuditLogEntry_1.AuditLogType.UserStateChange:
                this.blotter.api.auditEventApi._onAuditStateChanged.Dispatch(this.blotter, stateChangedArgs);
                break;
        }
    };
    return AuditLogService;
}());
exports.AuditLogService = AuditLogService;
