"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var NodeSchedule = require("node-schedule");
var StrategyConstants = require("../../Utilities/Constants/StrategyConstants");
var ArrayExtensions_1 = require("../Extensions/ArrayExtensions");
var DateExtensions_1 = require("../Extensions/DateExtensions");
/**
 * This class is used for managing scheduling of Reports and Reminders
 * It uses node-schedule (via a strongly typed NodeSchedule) and creates standard jobs
 * It also createsa daily job to run at midnight that will refresh the Blotter - this is so that date-based schedules can jump to the new day
 */
var ScheduleService = /** @class */ (function () {
    function ScheduleService(blotter) {
        var _this = this;
        this.blotter = blotter;
        this.blotter = blotter;
        this.blotter.adaptableBlotterStore.TheStore.subscribe(function () {
            return _this.listenToScheduleStoreChanges();
        });
        this.alertJobs = [];
        this.exportJobs = [];
        // create the midnight reload job
        var reloadSchedule = {
            Hour: 0,
            Minute: 1,
            DaysOfWeek: [0, 1, 2, 3, 4, 5, 6],
        };
        var date = this.getDateFromSchedule(reloadSchedule);
        if (date != null) {
            var refreshGridJob = NodeSchedule.scheduleJob(date, function () {
                _this.blotter.reloadGrid();
            });
        }
    }
    ScheduleService.prototype.listenToScheduleStoreChanges = function () {
        if (this.blotter.isInitialised) {
            if (this.reminderState != this.getReminderState()) {
                this.reminderState = this.getReminderState();
                var reminderStrategy = (this.blotter.strategies.get(StrategyConstants.ReminderStrategyId));
                reminderStrategy.scheduleReminders();
            }
            if (this.exportState != this.getExportState()) {
                this.exportState = this.getExportState();
                var exportStrategy = (this.blotter.strategies.get(StrategyConstants.ExportStrategyId));
                exportStrategy.scheduleReports();
            }
        }
    };
    ScheduleService.prototype.AddReminderSchedule = function (reminder) {
        var _this = this;
        var date = this.getDateFromSchedule(reminder.Schedule);
        if (date != null) {
            var alertJob = NodeSchedule.scheduleJob(date, function () {
                _this.blotter.api.alertApi.displayAlert(reminder.Alert);
            });
            this.alertJobs.push(alertJob);
        }
    };
    ScheduleService.prototype.AddReportSchedule = function (report) {
        var _this = this;
        if (report.AutoExport) {
            var date = this.getDateFromSchedule(report.AutoExport.Schedule);
            if (date != null) {
                var exportJob = NodeSchedule.scheduleJob(date, function () {
                    _this.blotter.api.exportApi.sendReport(report.Name, report.AutoExport.ExportDestination);
                });
                this.exportJobs.push(exportJob);
            }
        }
    };
    ScheduleService.prototype.getDateFromSchedule = function (schedule) {
        var date = null;
        if (schedule.OneOffDate != null) {
            date = new Date(schedule.OneOffDate);
            date.setHours(schedule.Hour);
            date.setMinutes(schedule.Minute);
            date.setSeconds(0);
        }
        else {
            date = new Date();
            if (ArrayExtensions_1.ArrayExtensions.ContainsItem(schedule.DaysOfWeek, date.getDay())) {
                date.setHours(schedule.Hour);
                date.setMinutes(schedule.Minute);
                date.setSeconds(0);
            }
            else {
                return null; // because it will rerun at midnight so we can get rid
            }
        }
        // add check for whether date in the past
        if (date != null && DateExtensions_1.DateExtensions.IsDateInFuture(date)) {
            return date;
        }
        return null;
    };
    ScheduleService.prototype.ClearAllAlertJobs = function () {
        this.alertJobs.forEach(function (j) {
            if (j != null) {
                j.cancel();
            }
        });
        this.alertJobs = [];
    };
    ScheduleService.prototype.ClearAllExportJobs = function () {
        this.exportJobs.forEach(function (j) {
            if (j != null) {
                j.cancel();
            }
        });
        this.exportJobs = [];
    };
    ScheduleService.prototype.getReminderState = function () {
        return this.blotter.api.reminderApi.getReminderState();
    };
    ScheduleService.prototype.getExportState = function () {
        return this.blotter.api.exportApi.getExportState();
    };
    return ScheduleService;
}());
exports.ScheduleService = ScheduleService;
