"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var StrategyConstants = require("../Constants/StrategyConstants");
var Enums_1 = require("../../PredefinedConfig/Common/Enums");
var StringExtensions_1 = require("../Extensions/StringExtensions");
var ArrayExtensions_1 = require("../Extensions/ArrayExtensions");
var LayoutHelper_1 = require("../Helpers/LayoutHelper");
var SearchService = /** @class */ (function () {
    function SearchService(blotter) {
        var _this = this;
        this.blotter = blotter;
        this.blotter.adaptableBlotterStore.TheStore.subscribe(function () { return _this.listenToSearchStoreChanges(); });
    }
    SearchService.prototype.listenToSearchStoreChanges = function () {
        var _this = this;
        if (this.blotter.isInitialised) {
            // Fire Search Changed for Advanced Search (if ServerSearchOption is not 'None')
            if (this.advancedSearchState != this.getAdvancedSearchState()) {
                this.advancedSearchState = this.getAdvancedSearchState();
                if (this.blotter.blotterOptions.generalOptions.serverSearchOption != 'None') {
                    this.publishSearchChanged(Enums_1.SearchChangedTrigger.AdvancedSearch);
                }
            }
            // Fire Search Changed for ColumnFilter (if ServerSearchOption is set too All Search)
            if (this.columnFilterState != this.getColumnFilterState()) {
                this.columnFilterState = this.getColumnFilterState();
                setTimeout(function () { return _this.blotter.applyGridFiltering(); }, 5);
                if (this.blotter.blotterOptions.generalOptions.serverSearchOption == 'AllSearch' ||
                    'AllSearchandSort') {
                    this.publishSearchChanged(Enums_1.SearchChangedTrigger.ColumnFilter);
                }
            }
            // Fire Search Changed for Data Source in all circumstances (as only makes sense on the server)
            if (this.dataSourceState != this.getDataSourceState()) {
                this.dataSourceState = this.getDataSourceState();
                this.publishSearchChanged(Enums_1.SearchChangedTrigger.DataSource);
            }
            // Fire Search Changed for Quick Search (if ServerSearchOption is set too All Search)
            if (this.quickSearchState != this.getQuickSearchState()) {
                this.quickSearchState = this.getQuickSearchState();
                // if not highlighting cell then lets tell quick search strategy to create a range
                if (this.quickSearchState.DisplayAction != Enums_1.DisplayAction.HighlightCell) {
                    var quickSearchStrategy = (this.blotter.strategies.get(StrategyConstants.QuickSearchStrategyId));
                    quickSearchStrategy.createQuickSearchRange();
                }
                this.blotter.applyGridFiltering();
                this.blotter.redraw();
                if (this.blotter.blotterOptions.generalOptions.serverSearchOption == 'AllSearch' ||
                    'AllSearchandSort') {
                    this.publishSearchChanged(Enums_1.SearchChangedTrigger.QuickSearch);
                }
            }
            // Fire Search Changed for User Filter (if ServerSearchOption is not 'None')
            if (this.userFilterState != this.getUserFilterState()) {
                this.userFilterState = this.getUserFilterState();
                setTimeout(function () { return _this.blotter.applyGridFiltering(); }, 5);
                if (this.blotter.blotterOptions.generalOptions.serverSearchOption != 'None') {
                    // we cannot stop all extraneous publishing (e.g. we publish if the changed user filter is NOT being used)
                    // but we can at least ensure that we only publish IF there are live searches or column filters
                    if (StringExtensions_1.default.IsNotNullOrEmpty(this.blotter.api.advancedSearchApi.getCurrentAdvancedSearchName()) ||
                        ArrayExtensions_1.default.IsNotNullOrEmpty(this.blotter.api.columnFilterApi.getAllColumnFilter())) {
                        this.publishSearchChanged(Enums_1.SearchChangedTrigger.UserFilter);
                    }
                }
                // Do Column Sorts and Columns Separately
                if (!ArrayExtensions_1.default.areArraysEqualWithOrderandProperties(this.columnSorts, this.getGridColumnSorts())) {
                    this.columnSorts = this.getGridColumnSorts();
                    if (this.blotter.blotterOptions.generalOptions.serverSearchOption == 'AllSearchandSort') {
                        this.publishSearchChanged(Enums_1.SearchChangedTrigger.Sort);
                    }
                    LayoutHelper_1.LayoutHelper.autoSaveLayout(this.blotter);
                }
                if (this.columns != this.getGridColumns()) {
                    this.columns = this.getGridColumns();
                    LayoutHelper_1.LayoutHelper.autoSaveLayout(this.blotter);
                }
            }
        }
    };
    SearchService.prototype.getAdvancedSearchState = function () {
        return this.blotter.api.advancedSearchApi.getAdvancedSearchState();
    };
    SearchService.prototype.getColumnFilterState = function () {
        return this.blotter.api.columnFilterApi.getColumnFilterState();
    };
    SearchService.prototype.getDataSourceState = function () {
        return this.blotter.api.dataSourceApi.getDataSourceState();
    };
    SearchService.prototype.getQuickSearchState = function () {
        return this.blotter.api.quickSearchApi.getQuickSearchState();
    };
    SearchService.prototype.getUserFilterState = function () {
        return this.blotter.api.userFilterApi.getUserFilterState();
    };
    SearchService.prototype.getGridColumns = function () {
        return this.blotter.api.gridApi.getColumns();
    };
    SearchService.prototype.getGridColumnSorts = function () {
        return this.blotter.api.gridApi.getColumnSorts();
    };
    /**
     * Each time any of the objects that make up search are changed (e.g. filters, quick search, advanced search, data sources etc.) we fire an event
     * This is primarily to help users who want to run search on the server and so need to know what has changed
     * @param searchChangedTrigger function that triggered the event
     */
    SearchService.prototype.publishSearchChanged = function (searchChangedTrigger) {
        if (this.blotter.isInitialised) {
            var currentDataSource = this.blotter.api.dataSourceApi.getCurrentDataSource();
            var currentAdvancedSearch = this.blotter.api.advancedSearchApi.getCurrentAdvancedSearch();
            // lets get the searchstate
            var blotterSearchState = {
                dataSource: currentDataSource == null ? null : currentDataSource,
                advancedSearch: currentAdvancedSearch == null ? null : currentAdvancedSearch,
                quickSearch: this.blotter.api.quickSearchApi.getQuickSearchValue(),
                columnFilters: this.blotter.api.columnFilterApi.getAllColumnFilter(),
            };
            var blotterSortState = {
                columnSorts: this.blotter.api.gridApi.getColumnSorts(),
                customSorts: this.blotter.api.customSortApi.getAllCustomSort(),
            };
            var searchChangedInfo = {
                searchChangedTrigger: searchChangedTrigger,
                blotterSearchState: blotterSearchState,
                blotterSortState: blotterSortState,
                searchAsAtDate: new Date(),
            };
            var searchEventData = {
                name: 'Adaptable Blotter',
                type: 'Search Args',
                id: searchChangedInfo,
            };
            var searchChangedArgs = {
                object: 'fdc3-context',
                definition: 'https://fdc3.org/context/1.0.0/',
                version: '1.0.0',
                data: [searchEventData],
            };
            this.blotter.api.eventApi._onSearchChanged.Dispatch(this.blotter, searchChangedArgs);
        }
    };
    return SearchService;
}());
exports.SearchService = SearchService;
